/*
*   Generating .mov from sequence of .ppm
*
*   (c) 23.8.2001 Ingo D. Rullhusen <d01c@uni-bremen.de>
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or   
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of 
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*   see gpl.txt for full license
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef OQT
#include <openquicktime.h>
#include <colormodels.h>
#else
#include <quicktime.h>
#endif


int main( int argc, char *argv[] )
{
   FILE *ppmfile;
   char buffer[32];
   int iw, ih, ic;
   quicktime_t *movfile;

   char ppmname[64], ppmformat[64], *movname, *compressor,
        *copyright, *name, *info;
   int width, height;
   float frame_rate;
   unsigned char **row_pointers;
   int first_frame, last_frame;

   char *pre, *suff;
   int i, zc, quality;

   pre = "pic";
   suff = ".ppm";
   zc = 10;
   first_frame = 0;
   last_frame = 0;
   frame_rate = 24.0;
   compressor = QUICKTIME_JPEG;
   movname = "pic.mov";
   info = "generated with makeqt";
   copyright = NULL;
   name = NULL;
   quality = 100;

   for ( i=0; i<argc; i++ )
   {
      if ( strcmp(argv[i],"-c") == 0 )
      {
         compressor = argv[i+1];
      }
      else if ( strcmp(argv[i],"-f") == 0 )
      {
         frame_rate = atof(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-fno" ) == 0 )
      {
         first_frame = atoi(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-lno" ) == 0 )
      {
         last_frame = atoi(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-o") == 0 )
      {
         movname = argv[i+1];
      }
      else if ( strcmp(argv[i],"-pre") == 0 )
      {
         pre = argv[i+1];
      }
      else if ( strcmp(argv[i],"-suff") == 0 )
      {
         suff = argv[i+1];
      }
      else if ( strcmp(argv[i],"-zc") == 0 )
      {
         zc = atoi(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-C") == 0 )
      {
         copyright = argv[i+1];
      }
      else if ( strcmp(argv[i],"-N") == 0 )
      {
         name = argv[i+1];
      }
      else if ( strcmp(argv[i],"-I") == 0 )
      {
         info = argv[i+1];
      }
      else if ( strcmp(argv[i],"-q") == 0 )
      {
         quality = atoi(argv[i+1]);
      }
      else if ( strcmp(argv[i],"-h") == 0 )
      {
         printf("usage: makeqt [options]\n");
         printf(" -h           this help\n");
         printf(" -c    COMP   set video compressor of libquicktime\n");
         printf(" -f    FLOAT  set frame rate\n");
         printf(" -fno  INT    set first frame number\n");
         printf(" -lno  INT    set last frame number\n");
         printf(" -o    NAME   set output file\n");
         printf(" -pre  STRING set prefix of ppm files\n");
         printf(" -suff STRING set suffix of ppm files\n");
         printf(" -zc   INT    set number of digits in frame number for file name\n");
         printf(" -C    STRING set copyright\n");
         printf(" -N    STRING set name\n");
         printf(" -I    STRING set info\n");
         printf(" -q    INT    set quality\n");
         return(0);
      }
   }

   sprintf(ppmformat,"%s%%0%ii%s",pre,zc,suff);
   sprintf(ppmname,ppmformat,first_frame);

   ppmfile = fopen( ppmname, "r" );
   if ( !ppmfile )
   {
      printf(" Cannot open %s!\n",ppmname);
      exit(1);
   }
   fscanf(ppmfile,"%s\n%i %i\n%i",buffer,&iw,&ih,&ic);
   if ( strcmp(buffer,"P6") != 0 || ic != 255 )
   {
      printf(" Input format not supported!\n");
      exit(1);
   }
   width = iw;
   height = ih;
   fclose(ppmfile);

   row_pointers = calloc( height, sizeof(unsigned char*) );
   row_pointers[0] = calloc( 3*width*height, sizeof(unsigned char) );
   for ( i=1; i<height; i++ )
   {
      row_pointers[i] = row_pointers[i-1] + 3*width;
   }

   movfile = quicktime_open(movname, 0, 1);

   if ( copyright ) quicktime_set_copyright( movfile, copyright );
   if ( name ) quicktime_set_name( movfile, name );
   if ( info ) quicktime_set_info( movfile, info );

   quicktime_set_video( movfile, 1, width, height, frame_rate, compressor);

   movfile->vtracks[0].track->mdia.mdhd.quality = quality;

   if ( quicktime_supported_video( movfile, 0) == 0 )
   {
      printf(" Encoder %s not supported!\n",compressor);
      exit(1);
   }

   for ( i=first_frame; i<last_frame+1; i++ )
   {
      sprintf(ppmname,ppmformat,i);

      ppmfile = fopen( ppmname, "rb" );
      if ( !ppmfile )
      {
         printf(" Cannot open %s!\n",ppmname);
         exit(1);
      }
      fscanf(ppmfile,"%s\n%i %i\n%i",buffer,&iw,&ih,&ic);
      if ( strcmp(buffer,"P6") != 0 || ic != 255
           || iw != width || ih != height )
      {
         printf(" Input format not supported!\n");
         exit(1);
      }
      fgetc(ppmfile);

      printf(" processing %s\n",ppmname);
   
      if ( fread( (void *)row_pointers[0],
                  3*sizeof(unsigned char)*width*height, 1, ppmfile )
           != 1 )
      {
         exit(1);
      }

#ifdef OQT
      if ( !( quicktime_writes_cmodel( movfile, BC_RGB888, 0 ) ) )
      {
         printf(" RGB24 not supported by encoder \"%s\"!\n",compressor);
         exit(1);
      }
      if ( quicktime_encode_video(  movfile, BC_RGB888, row_pointers, 0 ) )
#else
      if ( quicktime_encode_video(  movfile, row_pointers, 0 ) )
#endif
      {
         printf(" Unknown encoding error!\n");
         exit(1);
      }

      fclose(ppmfile);

   }

   quicktime_close(movfile);

   return(0);
}
